// Copyright (c) Microsoft and contributors.  All rights reserved.
// 
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
// http://www.apache.org/licenses/LICENSE-2.0
// 
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// 
// See the License for the specific language governing permissions and
// limitations under the License.
// 
// Code generated by Microsoft (R) AutoRest Code Generator.
// Changes may cause incorrect behavior and will be lost if the code is
// regenerated.

namespace Microsoft.Azure.Batch.Protocol.Models
{
    using System.Linq;

    /// <summary>
    /// The status of the Job Preparation and Job Release tasks on a compute
    /// node.
    /// </summary>
    public partial class JobPreparationAndReleaseTaskExecutionInformation
    {
        /// <summary>
        /// Initializes a new instance of the
        /// JobPreparationAndReleaseTaskExecutionInformation class.
        /// </summary>
        public JobPreparationAndReleaseTaskExecutionInformation() { }

        /// <summary>
        /// Initializes a new instance of the
        /// JobPreparationAndReleaseTaskExecutionInformation class.
        /// </summary>
        /// <param name="poolId">The id of the pool containing the compute
        /// node to which this entry refers.</param>
        /// <param name="nodeId">The id of the compute node to which this
        /// entry refers.</param>
        /// <param name="nodeUrl">The URL of the compute node to which this
        /// entry refers.</param>
        /// <param name="jobPreparationTaskExecutionInfo">Information about
        /// the execution status of the Job Preparation task on this compute
        /// node.</param>
        /// <param name="jobReleaseTaskExecutionInfo">Information about the
        /// execution status of the Job Release task on this compute
        /// node.</param>
        public JobPreparationAndReleaseTaskExecutionInformation(string poolId = default(string), string nodeId = default(string), string nodeUrl = default(string), JobPreparationTaskExecutionInformation jobPreparationTaskExecutionInfo = default(JobPreparationTaskExecutionInformation), JobReleaseTaskExecutionInformation jobReleaseTaskExecutionInfo = default(JobReleaseTaskExecutionInformation))
        {
            PoolId = poolId;
            NodeId = nodeId;
            NodeUrl = nodeUrl;
            JobPreparationTaskExecutionInfo = jobPreparationTaskExecutionInfo;
            JobReleaseTaskExecutionInfo = jobReleaseTaskExecutionInfo;
        }

        /// <summary>
        /// Gets or sets the id of the pool containing the compute node to
        /// which this entry refers.
        /// </summary>
        [Newtonsoft.Json.JsonProperty(PropertyName = "poolId")]
        public string PoolId { get; set; }

        /// <summary>
        /// Gets or sets the id of the compute node to which this entry refers.
        /// </summary>
        [Newtonsoft.Json.JsonProperty(PropertyName = "nodeId")]
        public string NodeId { get; set; }

        /// <summary>
        /// Gets or sets the URL of the compute node to which this entry
        /// refers.
        /// </summary>
        [Newtonsoft.Json.JsonProperty(PropertyName = "nodeUrl")]
        public string NodeUrl { get; set; }

        /// <summary>
        /// Gets or sets information about the execution status of the Job
        /// Preparation task on this compute node.
        /// </summary>
        [Newtonsoft.Json.JsonProperty(PropertyName = "jobPreparationTaskExecutionInfo")]
        public JobPreparationTaskExecutionInformation JobPreparationTaskExecutionInfo { get; set; }

        /// <summary>
        /// Gets or sets information about the execution status of the Job
        /// Release task on this compute node.
        /// </summary>
        /// <remarks>
        /// This property is set only if the Job Release task has run on the
        /// node.
        /// </remarks>
        [Newtonsoft.Json.JsonProperty(PropertyName = "jobReleaseTaskExecutionInfo")]
        public JobReleaseTaskExecutionInformation JobReleaseTaskExecutionInfo { get; set; }

        /// <summary>
        /// Validate the object.
        /// </summary>
        /// <exception cref="Microsoft.Rest.ValidationException">
        /// Thrown if validation fails
        /// </exception>
        public virtual void Validate()
        {
            if (this.JobPreparationTaskExecutionInfo != null)
            {
                this.JobPreparationTaskExecutionInfo.Validate();
            }
            if (this.JobReleaseTaskExecutionInfo != null)
            {
                this.JobReleaseTaskExecutionInfo.Validate();
            }
        }
    }
}
